/*
 * LoSTInterface.js
 * Yipeng Huang yh2315@columbia.edu
 * Columbia University
 * Function that conducts the LoST Query.
 * This enables the client to conduct cross-domain AJAX queries
 */

var lostQuery = function() {

    var pub = {};

    var useCase = "GEOLOCATION";

    //pointer to proxy script location, which must be in the same directory as LoSTInterface.js
    var PROXY_URL = "LoST_proxy.php";

    //pointer to lost server location
    var lostUrl;

    //service type
    var service;

    //recursive flag; default value is false unless set otherwise by user
    var recursive = false;

    //service boundary response type; default value is reference unless set otherwise by user
    var serviceBoundary = "reference";

    //locationId of the location profile
    var locationId;

    //associative array of geodetic-2d properties for geodetic location profiles
    var geodetic2D;

    //associative array of civic address propoerties for civic location profiles
    var civic;

    //whether or not the LoST server should validate the civic address
    var validateLocation = false;

    //whether or not the region flag should be set
    var region = false;

    // max distance to services
    var maxDistance;

    //maximum number of responses
    var limit;

    //public variables that will contain the response properties once query is successful
    pub.lat;
    pub.lon;

    pub.lostQueryString;
    pub.lostResponseString;

    pub.mappings;
    pub.locationValidation;

    //public variables that contain information about exceptions
    pub.exception = "";
    // pub.warnings;
    /*init checks to see if the set of options provided can lead to a valid query*/
    pub.init = function(options) {

        if (options) {

            if (options.lostUrl) {
                lostUrl = options.lostUrl;
            } else {
                pub.exception = pub.exception + "INIT EXCEPTION: LoST server URL missing.";
                // alert("LoST server URL missing");
                return false;
            }

            if (options.service) {
                service = options.service;
            } else {
                pub.exception = pub.exception + "INIT EXCEPTION: Service URN missing.";
                // alert("service URN missing");
                return false;
            }

            if (options.recursive && options.recursive === true) {
                recursive = true;
            }

            if (options.serviceBoundary && options.serviceBoundary == "value") {
                serviceBoundary = "value";
            }

            if (options.locationId) {
                locationId = options.locationId;
            }

            if (options.geodetic2D) {
                geodetic2D = options.geodetic2D;
                useCase = "LOCATION_PROFILE";
            }

            if (options.civic) {
                civic = options.civic;
                useCase = "LOCATION_PROFILE";
            }

            if (options.validateLocation && options.validateLocation === true) {
                validateLocation = true;
            }

            if (options.region && options.region === true) {
                region = true;
            }

            if (options.maxDistance) {
                maxDistance = options.maxDistance;
            }

            if (options.limit) {
                limit = options.limit;
            }
        }

        else {
            pub.exception = pub.exception + "INIT EXCEPTION: LoST query options incomplete.";
            // alert("ERROR: LoST query options incomplete.");
            return false;
        }
        return true;
    };

    /*query conducts the query using the parameters already passed to the object*/
    pub.query = function(success_callback, error_callback) {
	
        //Success callback function for geolocation
        geolocation_success_callback = function(p) {

            //Success callback function for lost query			
            lost_query_success_callback = function(p) {

                pub.lostResponseString = "<?xml" + p.responseText.split("<?xml")[1];

                /*query parse*/
                pub.mappings = parseMappings(pub.lostResponseString);
                pub.locationValidation = parseLocationValidation(pub.lostResponseString);

                /*return control*/
                success_callback();
            };

            //Error callback function for lost query
            lost_query_error_callback = function(p) {
                pub.exception = pub.exception + p.exception;
                error_callback();
            };

            pub.lat = p.coords.latitude.toFixed(3);
            pub.lon = p.coords.longitude.toFixed(3);

            /*generate query*/
            // If, according to the LoST extension, the user specifies a maximum distance, the program switches to Circle
            if (typeof maxDistance !== "undefined") {
                pub.lostQueryString = findServiceQueryByGeodetic2DCircle(recursive, serviceBoundary, limit, region, locationId, pub.lat, pub.lon, maxDistance, service);
            } else {
                pub.lostQueryString = findServiceQueryByGeodetic2DPoint(recursive, serviceBoundary, limit, region, locationId, pub.lat, pub.lon, service);
            }

            /*lost query*/
            sendQueryAJAX(PROXY_URL, lostUrl, pub.lostQueryString, lost_query_success_callback, lost_query_error_callback);

        };

        //Error callback function for lost query
        geolocation_error_callback = function(p) {
            // alert("GEOLOCATION EXCEPTION: " + p.message);
            pub.exception = pub.exception + "GEOLOCATION EXCEPTION: " + p.message;
            error_callback();
        };

        if (useCase == "GEOLOCATION") {

            if (typeof(navigator.geolocation) != "undefined") {
                navigator.geolocation.getCurrentPosition(geolocation_success_callback, geolocation_error_callback, {
                    maximumAge: 0
                });
            } else {
                pub.exception = pub.exception + "GEOLOCATION EXCEPTION: Browser does not support W3C geo.";
                // alert("ERROR: browser does not support W3C geo.");
                error_callback();
            }
        }

        else if (useCase == "LOCATION_PROFILE") {

            //Success callback function for lost query			
            lost_query_success_callback = function(p) {

                pub.lostResponseString = "<?xml" + p.responseText.split("<?xml")[1];

                /*query parse*/
                pub.mappings = parseMappings(pub.lostResponseString);
                pub.locationValidation = parseLocationValidation(pub.lostResponseString);

                /*return control*/
                success_callback();
            };

            //Error callback function for lost query
            lost_query_error_callback = function(p) {
                pub.exception = pub.exception + p.exception;
                error_callback();
            };

            /*generate query*/
            // Civic address
            if (typeof civic !== "undefined") {
                pub.lostQueryString = findServiceQueryByCivic(recursive, validateLocation, serviceBoundary, limit, region, locationId, civic, service);
            } else if (typeof geodetic2D !== "undefined") {
                //Geodetic-2D
                if (geodetic2D.shape == "Point") {
                    pub.lostQueryString = findServiceQueryByGeodetic2DPoint(recursive, serviceBoundary, limit, region, locationId, geodetic2D.pos.latitude, geodetic2D.pos.longitude, service);
                }
                else if (geodetic2D.shape == "Circle") {
                    pub.lostQueryString = findServiceQueryByGeodetic2DCircle(recursive, serviceBoundary, limit, region, locationId, geodetic2D.pos.latitude, geodetic2D.pos.longitude, geodetic2D.radius, service);
                }
                else if (geodetic2D.shape == "Ellipse") {
                    pub.lostQueryString = findServiceQueryByGeodetic2DEllipse(recursive, serviceBoundary, limit, region, locationId, geodetic2D.pos.latitude, geodetic2D.pos.longitude, geodetic2D.semiMajorAxis, geodetic2D.semiMinorAxis, geodetic2D.orientation, service);
                }
                else if (geodetic2D.shape == "ArcBand") {
                    pub.lostQueryString = findServiceQueryByGeodetic2DArcBand(recursive, serviceBoundary, limit, region, locationId, geodetic2D.pos.latitude, geodetic2D.pos.longitude, geodetic2D.innerRadius, geodetic2D.outerRadius, geodetic2D.startAngle, geodetic2D.openingAngle, service);
                }
                else if (geodetic2D.shape == "Polygon") {
                    pub.lostQueryString = findServiceQueryByGeodetic2DPolygon(recursive, serviceBoundary, limit, region, locationId, geodetic2D.posArray, service);
                }
                else {
                    pub.exception = pub.exception + "QUERY EXCEPTION: Unknown shape profile.";
                    // alert("ERROR: Unknown shape profile.");
                    error_callback();
                }
            } else {
                pub.exception = pub.exception + "QUERY EXCEPTION: Unknown location profile.";
                // alert("ERROR: Unknown location profile.");
                error_callback();
            }

            // alert(pub.lostQueryString);

            /*lost query*/
            sendQueryAJAX(PROXY_URL, lostUrl, pub.lostQueryString, lost_query_success_callback, lost_query_error_callback);

        }
    };
    return pub;
} ();