/*
 * QueryGenerator.js
 * Yipeng Huang yh2315@columbia.edu
 * Columbia University
 * Module that contains templates of LoST findService queries.
 */

// Create a query from template for civic locations
function findServiceQueryByCivic(recursive, validate_location, serviceBoundary, limit, region, locationId, civic, service) {
    var query = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n"
    + "<findService\r\n"
    + "    xmlns=\"urn:ietf:params:xml:ns:lost1\"\r\n";

    // Check to see if either of the LoST extension fields are present, if yes, add extension reference
    if (region || limit) {
        query = query + "    xmlns:ext=\"http://www.thisisnotdoneyet.net/lostNe\"\r\n";
    }

    // Add flags for recursive queries, for location validation, and how the service boundary should be returned.
    query = query + "    recursive=\"" + recursive + "\"\r\n"
    + "    validateLocation=\"" + validate_location + "\"\r\n"
    + "    serviceBoundary=\"" + serviceBoundary + "\">\r\n";

    // Add the limit quantity for LoST extension
    if (limit) {
        query = query + "    <ext:limit>" + limit + "</ext:limit>\r\n";
    }

    // Add the region flag for LoST extension
    if (region) {
        query = query + "    <ext:region>true</ext:region>\r\n";
    }

    // Add the civic location descriptor
    query = query + "    <location id=\"" + locationId + "\" profile=\"civic\">\r\n"
    + "        <civicAddress xmlns=\"urn:ietf:params:xml:ns:pidf:geopriv10:civicAddr\">\r\n";
    for (key in civic) {
        query = query + "            <" + key + ">" + civic[key] + "</" + key + ">\r\n";
    }
    query = query + "        </civicAddress>\r\n"
    + "    </location>\r\n";

    // Add service type
    query = query + "    <service>" + service + "</service>\r\n"
    + "</findService>\r\n";
    return query;
}

// Create a query from template for geodetic 2d point locations
function findServiceQueryByGeodetic2DPoint(recursive, serviceBoundary, limit, region, locationId, lat, lon, service) {    

    var query = findServiceQueryByGeodetic2DPrimitive(recursive, serviceBoundary, limit, region);

    // Add the geodetic 2d point location descriptor
    query = query + "    <location id=\"" + locationId + "\" profile=\"geodetic-2d\">\r\n"
    + "        <p2:Point id=\"point1\" srsName=\"urn:ogc:def:crs:EPSG::4326\">\r\n"
    + "            <p2:pos>" + lat + " " + lon + "</p2:pos>\r\n"
    + "        </p2:Point>\r\n"
    + "    </location>\r\n"
    + "    <service>" + service + "</service>\r\n"
    + "</findService>\r\n";

    return query;
}

// Create a query from template for geodetic 2d circle locations
function findServiceQueryByGeodetic2DCircle(recursive, serviceBoundary, limit, region, locationId, lat, lon, radius, service) {

    var query = findServiceQueryByGeodetic2DPrimitive(recursive, serviceBoundary, limit, region);

    // Add the geodetic 2d circle location descriptor
    query = query + "    <location id=\"" + locationId + "\" profile=\"geodetic-2d\">\r\n"
    + "        <p2:Circle srsName=\"urn:ogc:def:crs:EPSG::4326\">\r\n"
    + "            <p2:pos>" + lat + " " + lon + "</p2:pos>\r\n"
    + "            <p2:radius uom=\"urn:ogc:def:uom:EPSG::9001\">" + radius + "</p2:radius>\r\n"
    + "        </p2:Circle>\r\n"
    + "    </location>\r\n"
    + "    <service>" + service + "</service>\r\n"
    + "</findService>\r\n";
    return query;
}


// Create a query from template for geodetic 2d polygon locations
function findServiceQueryByGeodetic2DPolygon(recursive, serviceBoundary, limit, region, locationId, polygonPos, service) {

    var query = findServiceQueryByGeodetic2DPrimitive(recursive, serviceBoundary, limit, region);

    // Add the geodetic 2d polygon location descriptor
    query = query + "    <location id=\"" + locationId + "\" profile=\"geodetic-2d\">\r\n"
    + "        <p2:Polygon srsName=\"urn:ogc:def:crs:EPSG::4326\">\r\n"
    + "            <p2:exterior>\r\n"
    + "                <p2:LinearRing>\r\n";

    for (var i = 0; i < polygonPos.length; i++) {
        query = query + "                    <p2:pos>" + polygonPos[i].latitude + " " + polygonPos[i].longitude + "</p2:pos>\r\n";
    }

    query = query + "                 </p2:LinearRing>\r\n"
    + "             </p2:exterior>\r\n"
    + "        </p2:Polygon>\r\n"
    + "    </location>\r\n"
    + "    <service>" + service + "</service>\r\n"
    + "</findService>\r\n";
    return query;
}

// Create a query from template for geodetic 2d ellipse locations
function findServiceQueryByGeodetic2DEllipse(recursive, serviceBoundary, limit, region, locationId, lat, lon, semiMajorAxis, semiMinorAxis, orientation, service) {

    var query = findServiceQueryByGeodetic2DPrimitive(recursive, serviceBoundary, limit, region);

    query = query + "    <location id=\"" + locationId + "\" profile=\"geodetic-2d\">\r\n"

    // Add the geodetic 2d ellipse location descriptor
    + "        <p2:Ellipse srsName=\"urn:ogc:def:crs:EPSG::4326\">\r\n"
    + "            <p2:pos>" + lat + " " + lon + "</p2:pos>\r\n"
    + "            <p2:semiMajorAxis uom=\"urn:ogc:def:uom:EPSG::9001\">" + semiMajorAxis + "</p2:semiMajorAxis>\r\n"
    + "            <p2:semiMinorAxis uom=\"urn:ogc:def:uom:EPSG::9001\">" + semiMinorAxis + "</p2:semiMinorAxis>\r\n"
    + "            <p2:orientation uom=\"urn:ogc:def:uom:EPSG::9102\">" + orientation + "</p2:orientation>\r\n"
    + "        </p2:Ellipse>\r\n"
    + "    </location>\r\n"
    + "    <service>" + service + "</service>\r\n"
    + "</findService>\r\n";
    return query;
}

// Create a query from template for geodetic 2d arc band locations
function findServiceQueryByGeodetic2DArcBand(recursive, serviceBoundary, limit, region, locationId, lat, lon, innerRadius, outerRadius, startAngle, openingAngle, service) {

    var query = findServiceQueryByGeodetic2DPrimitive(recursive, serviceBoundary, limit, region);

    // Add the geodetic 2d arc band location descriptor
    query = query + "    <location id=\"" + locationId + "\" profile=\"geodetic-2d\">\r\n"
    + "        <p2:ArcBand srsName=\"urn:ogc:def:crs:EPSG::4326\">\r\n"
    + "            <p2:pos>" + lat + " " + lon + "</p2:pos>\r\n"
    + "            <p2:innerRadius uom=\"urn:ogc:def:uom:EPSG::9001\">" + innerRadius + "</p2:innerRadius>\r\n"
    + "            <p2:outerRadius uom=\"urn:ogc:def:uom:EPSG::9001\">" + outerRadius + "</p2:outerRadius>\r\n"
    + "            <p2:startAngle uom=\"urn:ogc:def:uom:EPSG::9102\">" + startAngle + "</p2:startAngle>\r\n"
    + "            <p2:openingAngle uom=\"urn:ogc:def:uom:EPSG::9102\">" + openingAngle + "</p2:openingAngle>\r\n"
    + "        </p2:ArcBand>\r\n"
    + "    </location>\r\n"
    + "    <service>" + service + "</service>\r\n"
    + "</findService>\r\n";
    return query;
}


function findServiceQueryByGeodetic2DPrimitive(recursive, serviceBoundary, limit, region) {

    var query = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n"
    + "<findService\r\n"
    + "    xmlns=\"urn:ietf:params:xml:ns:lost1\"\r\n"
    + "    xmlns:p2=\"http://www.opengis.net/gml\"\r\n";


    // Check to see if either of the LoST extension fields are present
    if (region || limit) {
        query = query + "    xmlns:ext=\"http://www.thisisnotdoneyet.net/lostNe\"\r\n";
    }

    // Add flags for recursive queries, for location validation, and how the service boundary should be returned.
    query = query + "    serviceBoundary=\"" + serviceBoundary + "\"\r\n"
    + "    recursive=\"" + recursive + "\">\r\n";

    // Add the limit quantity for LoST extension
    if (limit) {
        query = query + "    <ext:limit>" + limit + "</ext:limit>\r\n";
    }

    // Add the region flag for LoST extension
    if (region) {
        query = query + "    <ext:region>true</ext:region>\r\n";
    }

    return query;

}